/*****************************************************************/
/*
/* create1b.c copyright (c) 1994 Clemson University, all rights reserved.
/*
/* CREATE1B .C - Main calling program.
/*  Default usage: 
/*   create1b -i <input_file> 
/*  Options:  
/*   -n specifies the Ness107_file
/*   -o specifies output file name (default is "1b.dat")
/*   -c turns off calibration functions (default is on).
/*   -d allows a marginal level debugging capability.
/*   -g turns off georegistration functions (default is on).
/*   -u specifies unpacked channels and the low 5 bits of the value indicates 
/*      which channels are active.  eg 00011111 = 31 = all 5 on.
/*                                  eg 00010011 = 19 = ch 1,4,5.
/*      Note for -u the output file name is appended by ".<channel>.dat"
/*   -v replaces the program corrections with the errors found in the VAX code 
/*      so that EXACT replication of output exists. (default uses corrections)
/*   -x specifies multiple number of processors runnimg the program (default 1)
/*   -y specifies the processor ID (0 - X-1) (default is 0)
/*   -r specifies raw data (no headers)
/*   -h displays this options list.
/*
/* $Log: create1b.c,v $
 * Revision 1.2  1995/06/07  13:31:30  rheil
 * Semi Stable version for alpha, sparc, hp, and pentium.  Multiple processors.
 *
 * Revision 1.1  1995/02/28  17:12:54  rheil
 * Initial revision
 *
/*  9/1/1994 - 1/1/1995  - Code Origination, Robert M. Heil (Bob)
/*  1/10/1995 - Support SPARC as well as ALPHA, can run multiple processors.
/*
/******************************************************************/
/* UNIX INCLUDE FILES */
#include <stdio.h>
 
/* CREATE_1B INCLUDE FILES */
#include "macros.h"
#include "types.h"
#include "hrpt.h"
#include <sys/time.h>

/* GLOBAL VARIABLES */
static char create1b_c[] = "$Header: /projects/level1b/create.1bl/C/RCS/create1b.c,v 1.2 1995/06/07 13:31:30 rheil Exp $";
extern int4 optind;       /*** used with getopt for argument processing ***/
extern byte *optarg;      /*** used with getopt for argument processing ***/

/* FUNCTIONS CALLED */
/* void gen_header(hrpt_header, runrec, int2)
/* void gen_output(hrpt_header, FILE, FILE, runrec, outrec, rmc)
/* void gen_tbm(hrpt_header, runrec, rmc, byte)
/* void georegister(hrpt_header, runrec)
/* void initcal(FILE, hrpt_header, runrec, outrec, rmc)
/* void initheader(FILE, hrpt_header, runrec)
/* void initscan(FILE, hrpt_header, runrec)
/******************************************************************/

main(argc, argv)
int4 argc;
byte **argv;
{
     hrpt_header header;
     runrec      rundata;
     outrec      outdata;
     rmc         calibration;
     FILE        *hrpt_file;
     FILE        *oneb_file[5];
     byte        tbm[122];
     int2        data_hdr[3700],temp[1];
     int4        i;
     int8        start,end,init_s,init_tot,gen_s,gen_tot;
     struct      timeval tp;
     struct      timezone tzp ;

     tp.tv_sec = tp.tv_usec = end = init_tot = gen_tot = 0;
     GETTIMEOFDAY(&tp,&tzp);
     start = tp.tv_sec;

     bzero(&rundata,sizeof(rundata));
     strcpy(rundata.outfile[0],"1b.dat"); /* Set up default output */
     strcpy(rundata.ness,"n107");
     strcpy(rundata.nessbin,"n107.bin");
     /* Parse options from command line and open files */
     while (( i = getopt (argc, argv, "i:n:o:gcdu:vx:y:rzh")) != EOF)
          switch (i)
          {
          case 'i':
              if (rundata.i_flag++)
                 rundata.err_flag++;
              if ((hrpt_file = fopen(optarg,"r")) == NULL)
              {
                 rundata.err_flag++;
                 fprintf(stderr,"Error: cannot open file: %s\n",optarg);
              }
              break;
          case 'n':
              if (rundata.n_flag++)
                 rundata.err_flag++;
              strcpy(rundata.ness,optarg);
              strcpy(rundata.nessbin,optarg);
              strcat(rundata.nessbin,".bin");
              break;
          case 'o':
              if (rundata.o_flag++)
                 rundata.err_flag++;
              strcpy(rundata.outfile[0],optarg);
              break;
          case 'g':
              if (rundata.g_flag++)
                 rundata.err_flag++;
              break;
          case 'c':
              if (rundata.c_flag++)
                 rundata.err_flag++;
              break;
          case 'd':
              if (rundata.d_flag++)
                 rundata.err_flag++;
              break;
          case 'u':
              rundata.u_flag = atoi(optarg);
              break;
          case 'v':
              if (rundata.v_flag++)
                 rundata.err_flag++;
              break;
          case 'x': /* Total number of processors running the program */
              rundata.x_flag = atoi(optarg);
              break;
          case 'y': /* Processor ID out of x processors */
              rundata.y_flag = atoi(optarg);
              break;
          case 'r': /* Process raw data only */
              rundata.r_flag = 1;
              rundata.err_flag++;
              fprintf(stderr,"Error: -r option not implemented\n");
              break;
          case 'z': /* Fill data gaps with black line */
              rundata.z_flag = 1;
              break;
          case 'h':
              fprintf(stderr,"\nDefault usage: create1b -i <input_file>\n\n");
              fprintf(stderr,"Options:\n     -o specifies output file name ");
              fprintf(stderr,"(default is '1b.dat')\n");
              fprintf(stderr,"     -n <Ness107_ascii_file_name>  ");
              fprintf(stderr,"(default is 'n107')\n");
              fprintf(stderr,"     -x the number of processors running ");
              fprintf(stderr,"(default is 1).\n");
              fprintf(stderr,"     -y the ID of this processor         ");
              fprintf(stderr,"(default is 0).\n");
              fprintf(stderr,"     -g turns off georegistration functions.\n");
              fprintf(stderr,"     -c turns off calibration     functions.\n");
              fprintf(stderr,"     -d allows a marginal level debugging.\n");
              fprintf(stderr,"     -r dumps raw data files (no headers).\n");
              fprintf(stderr,"     -z Fills data gaps with black line.\n");
              fprintf(stderr,"     -u specifies unpacked channels:\n");
              fprintf(stderr,"        eg 00011111 = 31 = ch 1,2,3,4,5.\n");
              fprintf(stderr,"        eg 00010011 = 19 = ch 1,4,5.\n");
              fprintf(stderr,"        Note'.<ch>.dat' added to output files\n");
              fprintf(stderr,"     -v replaces the program corrections with");
              fprintf(stderr," VAX code errors.\n\n");
              exit(1);
              break;
          }

     if (rundata.u_flag)  /* If desire to output unpacked data channels */
     {
         /* Eliminate all but 5 lowest bits in u_flag */
         rundata.u_flag &= 0x01f;
         /* Starting with channel 1 and up to channel 5, see if on or off */
         for(i=0;i<5;i++)
             if ((rundata.u_flag >> (4-i)) & 0x1)  /* if ON */
             {
                 /* Set up the next file name (in case needed) */ 
                 strcpy(rundata.outfile[rundata.channels+1],
                        rundata.outfile[rundata.channels]);
                 /* Append the .channel.dat to the file name  */
                 sprintf(rundata.outfile[rundata.channels],"%s.%d.dat",
                         rundata.outfile[rundata.channels],i+1);
                 /* Keep track of the number of channels being written */
                 rundata.channels++;
             }
         if (!rundata.channels) /* If no channels requested?  */
         {
             rundata.err_flag++;
             fprintf(stderr,"Error: Bad value for -u option.\n");
         }
         for(i=0;i<rundata.channels;i++)
         {
             printf("%d = %s\n",i,rundata.outfile[i]);
             if ((oneb_file[i] = fopen(rundata.outfile[i],"w")) == NULL)
             {
                 rundata.err_flag++;
                 fprintf(stderr,"Error: cannot open: %s\n",rundata.outfile[i]);
             }
         }
     }
     else  /* No unpacked option */
     {
         rundata.channels = 1;
         if ((oneb_file[0] = fopen(rundata.outfile[0],"w")) == NULL)
         {
             rundata.err_flag++;
             fprintf(stderr,"Error: cannot open file: %s\n",rundata.outfile[0]);
         }
     }

     if (!rundata.x_flag)  /* If no multiple processors designated */
        rundata.x_flag = 1;
     if (!rundata.y_flag)  /* If no processor ID designated */
        rundata.y_flag = 0;
     if ((rundata.y_flag) >= (rundata.x_flag))  
     {
         rundata.err_flag++;
         fprintf(stderr,"Error: Processor ID larger that number of CPUs\n");
     }

     /* If errors present or no input file or no ness107 file specified */
     if (rundata.err_flag || !rundata.i_flag)
     {
        fprintf(stderr,"Usage: create1b -i <input_HRPT_file> ");
        fprintf(stderr,"[-n <NESS107_file>] [-o <output_file>]\n");
        fprintf(stderr,"  [-c] [-d] [-g] [-u <channel>] [-x <# of CPU's>]");
        fprintf(stderr,"[-y <CPU ID #>] [-v] [-r] [-h(elp)]\n");
        exit(1);
     }

     /* Initialization */
     GETTIMEOFDAY(&tp,&tzp);
     init_s = tp.tv_usec;
     initheader(hrpt_file,&header,&rundata);
     initscan(hrpt_file,&header,&rundata);
     georegister(&header, &rundata);
     initcal(hrpt_file,&header,&rundata,&outdata,&calibration);
     GETTIMEOFDAY(&tp,&tzp);
     init_tot = tp.tv_usec - init_s;
     if (tp.tv_usec - init_s < 0)
         init_tot += 998448; /* The largest value in the cycle */


     /* Generate and write the TBM Header */
     bzero(tbm,sizeof(tbm));
     gen_tbm(&header,&rundata,&calibration,tbm);
     for(i=0;i<rundata.channels;i++)
         fwrite(tbm, sizeof(tbm),1,oneb_file[i]);

     /* Generate and write the first data set header */
     bzero(data_hdr,sizeof(data_hdr));
     gen_header(&header,&rundata,data_hdr);
     BYTE_SWAP(data_hdr,2,3700);
     for(i=0;i<rundata.channels;i++)
         fwrite(data_hdr, sizeof(data_hdr),1,oneb_file[i]);
     /* Added dummy header 5/5/95 */
     bzero(data_hdr,sizeof(data_hdr));
     for(i=0;i<rundata.channels;i++)
         fwrite(data_hdr, sizeof(data_hdr),1,oneb_file[i]);

     /* Process Output scan line by scan line */
     for(i=0;i<5;i++)
        outdata.filler[i] = 0;
     for(i=0;i<88;i++)
        outdata.spare[i] = 0;
     if(rundata.c_flag)
     {
        printf("Omitting calibration information!\n");
        outdata.isg1 = outdata.isg2 = outdata.isg3 = outdata.isg4 = 0;
        outdata.isg5 = outdata.is1 = outdata.is2 = outdata.is3 = 0;
        outdata.is4 = outdata.is5 = 0;
     }
     if(rundata.g_flag)
     {
        printf("Omitting georegistration information!\n");
        outdata.igood = 0;
        for(i=0;i<51;i++)
           outdata.iearth[i] = 0;
        for(i=0;i<51;i++)
           outdata.izen[i] = 0;
        for(i=0;i<30;i++)
           outdata.itelm[i] = 0;
     }
     GETTIMEOFDAY(&tp,&tzp);
     gen_s = tp.tv_sec;
     gen_output(hrpt_file,&oneb_file[0],&header,&rundata,&outdata,&calibration);

     /* Fix the ngaps....  */
     printf("NGAPS = %d\n",rundata.ngaps);
     temp[0]=((rundata.ngaps >> 8) & 0xff) | ((rundata.ngaps & 0xff) << 8);
     for(i=0;i<rundata.channels;i++)
     {
         fseek(oneb_file[i],122+24,0);
         fwrite(temp,sizeof(temp),1,oneb_file[i]);
     }

     GETTIMEOFDAY(&tp,&tzp);
     gen_tot = tp.tv_sec - gen_s;
     end = tp.tv_sec - start;
     if(start) /* If there is timing to be printed out... */
     {
          printf("gen_output(msec) = %6d\n",gen_tot*1000+tp.tv_usec/1000);
          printf("Initial   (msec) = %6d\n",init_tot/1000);
          printf("Cum total (msec) = %6d    ",end*1000);
          printf("Secs %d\n\n",end);
     }
}
