/*****************************************************************/
/*
/* gen_output.c copyright (c) 1994 Clemson University, all rights reserved.
/*
/* GEN_OUTPUT.C - Generate sequential output frames
/*
/*    Routine to process satellite calibration, earth location, zenith angles,
/*    scan time, scan number, pack video, pack telemetry, and pack
/*    quality bits in 1b output format consistent with create1b.f sec 3
/*
/* $Log: gen_output.c,v $
 * Revision 1.3  1996/03/07  20:45:59  walt
 * added missing code for get_target and get_space
 *
 * Revision 1.2  1995/06/07  13:31:30  rheil
 * Semi Stable version for alpha, sparc, hp, and pentium.  Multiple processors.
 *
 * Revision 1.1  1995/02/28  17:12:54  rheil
 * Initial revision
 *
/*  1/1/1994  - Code Origination, Robert M. Heil (Bob)
/*  1/10/1995 - Support SPARC as well as ALPHA, can run multiple processors.
/*
/******************************************************************/
/* UNIX INCLUDE FILES */
#include <stdio.h>
 
/* CREATE_1B INCLUDE FILES */
#include "macros.h"
#include "types.h"
#include "hrpt.h"
#include <sys/time.h>

/* GLOBAL VARIABLES */
static char gen_output_c[] = "$Header: /projects/level1b/create.1bl/C/RCS/gen_output.c,v 1.3 1996/03/07 20:45:59 walt Exp $";

/* FUNCTIONS CALLED */
/* void browerlydanne(flt8, flt8) 
/* void byte_swap(byte, int4, int4)
/* void earth_pack(flt8, flt8, outrec)
/* void earthlocate(flt8, flt8, int4, flt8, flt8, flt8)
/* void filtp(runrec, rmc)
/* int4 get_scandate(hrpt_scanline, int2)
/* int4 get_scantime(hrpt_scanline, int2)
/* void quality_pack(hrpt_scanline, hrpt_header, runrec, rmc, qual)
/* void radiance(runrec, rmc)
/* void telem_pack(int2, pack_3)
/* flt8 scanorient(runrec, int4)
/* void space_intern(hrpt_scanline, runrec, int4, flt4, flt4)
/* void video_pack(int2, pack_3)
/* void zenith(flt8, flt8, flt8, int4, int4, int4, int4)
/******************************************************************/
 
void gen_output(hrpt_file, oneb_file,header,rundata,outdata,cal)
hrpt_header *header;
FILE        *hrpt_file;
FILE        *oneb_file[5];
runrec      *rundata;
outrec      *outdata;
rmc         *cal; 
{
  hrpt_scanline scanline;
  int4 start, finish, time, date, oldtime, olddate, deltime, deldate;
  int4 i, count, rec, ready, stack, channel, tindex, sindex;
  flt4 tar[3], sp[3], ave10_t[3][10], ave10_s[3][10];
  flt4 deltar, delsp, tstack[3][5], sstack[3][5], sum1, sum2;
  flt8 timediff, phig, vke[9],lat[LEN],lon[LEN], zen[LEN];
  int4 stamp,stamp2,scan_num,ngaps,miscn,total_w=0;
  int8 read_s,read_tot,write_s,write_tot,byte_s,byte_tot;
  byte temp[14800];
 
  struct timeval tp;
  struct timezone tzp;

  bzero(temp,sizeof(temp));
  for (channel=0;channel<3;channel++)
      for (i=0;i<5;i++)
      {
          tstack[channel][i] = rundata->tarave[channel];
          sstack[channel][i] = rundata->spave[channel];
      }
  oldtime = olddate = ready = stack = tindex = sindex = scan_num = ngaps = 0;

  if (rundata->y_flag == 0)
     start = rundata->firstscan;
  else
     start = (rundata->nscans / rundata->x_flag) * rundata->y_flag;

  if (rundata->y_flag == rundata->x_flag - 1)
     finish = rundata->nscans;
  else
     finish = ((rundata->nscans / rundata->x_flag) * (rundata->y_flag + 1)) - 1;

  if(rundata->d_flag)
  {
    printf("x =     %d y      = %d\n",rundata->x_flag,rundata->y_flag);
    printf("start = %d finish = %d\n",start,finish);
  }
  fseek(hrpt_file, HDRSZ + SCANSZ * (rundata->firstscan + 
                   (rundata->nscans / rundata->x_flag) * rundata->y_flag),0);

  /* Initialize timing variables */
  read_s = read_tot = write_s = write_tot = byte_s = byte_tot = 0;
  tp.tv_usec = 0;

  if(rundata->d_flag) 
  {
    fprintf(stderr,"[                     ] (done?)");
    for(i = 0; i<30;i++)
        fprintf(stderr,"%c",8);  /* Backup the cursor for % complete display */
    fflush(stderr);
  }
  for (rec = start;rec < finish; rec++)
  {
    GETTIMEOFDAY(&tp,&tzp);
    read_s = tp.tv_usec;
    fread(&scanline,SCANSZ,1,hrpt_file);
    GETTIMEOFDAY(&tp,&tzp);
    read_tot += tp.tv_usec - read_s;
    if (tp.tv_usec - read_s < 0)
        read_tot += 998448; /* The largest value in the cycle */

    /* Qualify scan times to ensire a good scan */
    time = get_scantime(&scanline,0);
    date = get_scandate(&scanline,0);
    deltime = time - oldtime;
    deldate = date - olddate;
    if (deldate == 0 && (deltime == 166 || deltime == 167)) /* good scan */
    {
      /* ---------------------------------------------------------------*/
      /* Swap the word values for entire scanline */
      BYTE_SWAP(&scanline.pn_count,2,11008);

      if(!rundata->c_flag)
      {
         /* Computing calibration factors */
         /* ---------------------------------------------------------------*/
         /* PRT value from minor frame header */
         if (stack == 10) stack = 0;
         if (scanline.doc.telemetry[2] <= 10)
         {
            rundata->prt[0][stack] = 1;
            ready = 1;
            count = 0;
         }
         if (ready && (scanline.doc.telemetry[2] > 10) && count < 4)
            rundata->prt[++count][stack]=scanline.doc.telemetry[2];
         if (ready && count == 4) /* Can immediately increment stack! */
         {
            ++stack;
            ready = 0;
         }

         /* Internal Target data & Space view counts */
         space_intern(&scanline,rundata,stack,tar,sp);
         for (channel = 0; channel < 3; channel++)
         {
             ave10_t[channel][stack-1] = tar[channel];
             ave10_s[channel][stack-1] = sp[channel];
         }

	 /* Extract space and target data (formerly get_target and get_space) */
	for (channel = 0; channel < 3; channel++)
	{
		flt4 tsum,ssum;
		int4 tcnt,scnt;
		tsum = 0.0;
		ssum = 0.0;
		for (i = 0; i < 10; i++)
		{
			tsum += scanline.doc.internal[i].ch[channel];
			ssum += scanline.doc.space[i].ch[channel];
			}
		tar[channel] = tsum / 10;
		sp[channel] = tsum / 10;
		tsum = 0.0;
		ssum = 0.0;
		tcnt = 0;
		scnt = 0;
		for (i = 0; i < 10; i++)
		{
			flt4 delta;
			delta = tar[channel] - scanline.doc.internal[i].ch[channel];
			if (delta < 0.0) delta *= -1;
			if (delta <= 100.0)
			{
				tsum += scanline.doc.internal[i].ch[channel];
				tcnt++;
				}
			delta = sp[channel] - scanline.doc.space[i].ch[channel];
			if (delta < 0.0) delta *= -1;
			if (delta <= 100.0)
			{
				ssum += scanline.doc.space[i].ch[channel];
				scnt++;
				}
			}
		if (tcnt >= 1) tar[channel] = tsum / tcnt;
		if (scnt >= 1) sp[channel] = ssum / scnt;
		}
         /* Update reference stacks (formerly AVE50) */
         for (channel = 0; channel < 3; channel++)
         {
             deltar = rundata->tarave[channel] - tar[channel];
             if (deltar < 0) deltar *= -1;
             if (deltar <= 100)
                tstack[channel][tindex] = tar[channel];
             else
                tstack[channel][tindex] = rundata->tarave[channel];
             if (++tindex == 5) tindex = 0;
             delsp  =  rundata->spave[channel] - sp[channel];
             if (delsp  < 0) delsp  *= -1;
             if (delsp <= 100)
                sstack[channel][sindex] = sp[channel];
             else
                sstack[channel][sindex] = rundata->spave[channel];
             if (++sindex == 5) sindex = 0;
         }
         for (channel = 0; channel < 3; channel++)
         {
             sum1 = sum2 = 0;
             for (i = 0; i < 5; i++)
             {
                 sum1 += tstack[channel][i];
                 sum2 += sstack[channel][i];
             }
             rundata->tarave[channel] = sum1/5;
             rundata->spave[channel] = sum2/5;
         }

         /*  Internal Temperature */
         filtp(rundata,cal);
         
         /*  Internal Target Radiance */
         radiance(rundata,cal);
      }

      if(!rundata->g_flag)
      {
         /* Earth location and zenith angle routines */
         /* ---------------------------------------------------------------*/
         phig = scanorient(rundata,time);
         timediff = date * 86400.0 + time/1000.0 - rundata->epochsec;
         if (timediff< 0 && date == 365) timediff += 31536000.0;
         browerlydanne(timediff,vke);
         earthlocate(phig,vke,nsig,sigv,lat,lon);
         /* In original VAX code the Lat and Lon parameters were swapped */
         if(rundata->v_flag)
            zenith(lon,lat,zen,time,date,oldtime,olddate);
         else /* this is the proper order */
            zenith(lat,lon,zen,time,date,oldtime,olddate);
      }

      /* Scan Line Number */
      miscn = rec - ++scan_num - ngaps; /* Number of missing scans */
      ngaps = rec - scan_num;           /* Cumulative missing scans*/

      /* Collect the telemetry */
      telem_pack(scanline.sync,outdata->itelm);

      /* Quality bit check */
      quality_pack(&scanline, header, rundata, cal, &outdata->iqual);

      if(!rundata->c_flag)
      {
        /* PACK (and scale) Calibration factors  */
        for (i=2;i<5;i++)
        {
            rundata->scaled_xint[i] = cal->xint[i] * 4194304.2;
            rundata->scaled_gain[i] = cal->gain[i] * 1073741824.2;
        }
        BYTE_SWAP(&rundata->scaled_xint[2],4,3);
        BYTE_SWAP(&rundata->scaled_gain[2],4,3);
        outdata->isg3 = rundata->scaled_gain[2];
        outdata->isg4 = rundata->scaled_gain[3];
        outdata->isg5 = rundata->scaled_gain[4];
        outdata->is3 = rundata->scaled_xint[2];
        outdata->is4 = rundata->scaled_xint[3];
        outdata->is5 = rundata->scaled_xint[4];
      }

      if(!rundata->g_flag)
      {
        /* PACK (and scale) Earth location */
        earth_pack(lat,lon,outdata);

        /* PACK (and scale) Zenith angles */
        for (i=0;i<51;i++)
            outdata->izen[i] = (int) (zen[i] * 2.0);
      }

      /* PACK transfer time code */
      outdata->jdat = ((header->pass_year & 0x007f)    <<  1) | 
                      ((scanline.doc.time[0] & 0x0100) >>  8) | 
                      ((scanline.doc.time[0] & 0x00ff) <<  8);
      outdata->jtim = ((scanline.doc.time[1] & 0x0070) >>  4) |
                      ((scanline.doc.time[1] & 0x000f) << 12) |
                      ((scanline.doc.time[2] & 0x03c0) <<  4) |
                      ((scanline.doc.time[2] & 0x003f) << 18) |
                      ((scanline.doc.time[3] & 0x0300) <<  8) |
                      ((scanline.doc.time[3] & 0x00ff) << 24); 
      /*
      stamp2 = stamp;
      stamp = ((scanline.doc.time[1] &0xffff) <<26) | 
              ((scanline.doc.time[2] &0xffff) <<10) | 
               (scanline.doc.time[3] & 0x3ff);
      printf("t[0] = %x  ",scanline.doc.time[0]);
      printf("t[1] = %x  ",scanline.doc.time[1]);
      printf("t[2] = %x  ",scanline.doc.time[2]);
      printf("t[3] = %x  ",scanline.doc.time[3]);
      printf("jtim = %x\n",outdata->jtim);
      printf("time stamp  = %x  |  ",stamp);
      printf(" diff = %d\n",stamp-stamp2);
      if((stamp-stamp2) > 167 | (stamp-stamp2) < 166 ) printf("   gap\n");
      */

      BYTE_SWAP(&outdata->jdat,2,1);   /* fix 2 word lengths */
      BYTE_SWAP(&outdata->jtim,4,1);   /* fix 4 word lengths */

      /* PACK scan line number */
      outdata->inum = ((scan_num & 0xff00) >>8) | ((scan_num & 0x00ff) <<8);
      BYTE_SWAP(&outdata->inum,2,1); /* fix 2 word lengths */

      /* PACK video data  */
      GETTIMEOFDAY(&tp,&tzp);
      byte_s = tp.tv_usec;
      video_pack(scanline.video,outdata->iscan);
      GETTIMEOFDAY(&tp,&tzp);
      byte_tot += tp.tv_usec - byte_s;
      if (tp.tv_usec - byte_s < 0)
          byte_tot += 998448; /* The largest value in the cycle */

      /* WRITE video data to file */
      BYTE_SWAP(outdata->iearth,4,3586);   /* fix all 4 word lengths */

      for(i=0;i<rundata->channels;i++)
      {
          GETTIMEOFDAY(&tp,&tzp);
          write_s = tp.tv_usec;
          fwrite(outdata, 1, OUTSZ, oneb_file[i]);
          GETTIMEOFDAY(&tp,&tzp);
          write_tot += tp.tv_usec - write_s;
          if (tp.tv_usec - write_s < 0)
              write_tot += 998448; /* The largest value in the cycle */
      }
      total_w++;

    } 
    else /* bad scan */
    {
      rundata->ngaps++;
      if(rundata->z_flag)
      {
          for(i=0;i<rundata->channels;i++)
               fwrite(temp,1,sizeof(temp), oneb_file[i]);
          total_w++;
      }
    }
    oldtime = time;
    olddate = date;
    if(rundata->d_flag) 
         if(!(rec % (rundata->nscans/(20 * rundata->x_flag))))
              fprintf(stderr,"*");
    fflush(stderr);
  }
  if(rundata->d_flag) 
       fprintf(stderr,"\n");
  fprintf(stderr,"num_gaps = %d total written = %d\n",rundata->ngaps, total_w);

  if(write_s) /* if > 0 then code to do timing is implemented */
  {
    printf("Write tot (msec) = %6d\n",write_tot/1000);
    printf("Read tot  (msec) = %6d\n",read_tot/1000);
    printf("Byte tot  (msec) = %6d\n",byte_tot/1000);
  }

  /* Write end of file marks */
}
