/*****************************************************************/
/*
/* gen_raw.c copyright (c) 1995 Clemson University, all rights reserved.
/*
/* GEN_RAW.C - Generate raw output from 1b data set.
/*
/*    Routine to extract 1b data and write to raw file. Data sets can be
/*    interleaved, or not, and can be 8 or 16 bit information. A header file
/*    is generated that contains metadata from the 1b file.
/*
/* $Log: gen_raw.c,v $
 * Revision 1.1  1995/06/07  13:31:30  rheil
 * Initial revision
 *
/*
/******************************************************************/
/* UNIX INCLUDE FILES */
#include <stdio.h>
 
/* CREATE_1B INCLUDE FILES */
#include "macros.h"
#include "types.h"
#include "hrpt.h"

/* GLOBAL VARIABLES */
static char gen_raw_c[] = "$Header: /projects/level1b/create.1bl/C/RCS/gen_raw.c,v 1.1 1995/06/07 13:31:30 rheil Exp $";
#define COLS 683
#define L1BHDR 122+7400+7400
#define VIDEO_OFFSET 448
#define TIME_OFFSET 4
#define RECORD_SZ  14800
#define VIDEO_SEEK L1BHDR+VIDEO_OFFSET
#define TIME_SEEK  L1BHDR+TIME_OFFSET

/* FUNCTIONS CALLED */
int4 summ();
void fill_in();
/******************************************************************/
 
void gen_raw(data)
onebrec *data;
{
 
     int i, j, k;
     unpack_5 unpack[COLS];
     int2 recs,gaps;
     int4 time;
     char temp[4];

     /* Extract the number of records from the data header: byte 130-131 */
     fseek(data->oneb_f, 130,0);
     fread(&temp[1],1,1,data->oneb_f);
     fread(&temp[0],1,1,data->oneb_f);
     recs = (((temp[1] & 0xff) << 8) | (temp[0] & 0xff));

     /* Extract the number of data gaps from the data header: byte 146-147 */
     fseek(data->oneb_f, 146,0);
     fread(&temp[1],1,1,data->oneb_f);
     fread(&temp[0],1,1,data->oneb_f);
     gaps = (((temp[1] & 0xff) << 8) | (temp[0] & 0xff));
     fprintf(stderr,"       *From %s (header): %d records, %d gaps",
             data->oneb_n, recs,gaps);
     fprintf(stderr,"=> %d out\n",recs-gaps);

     /* Extract the time stamp from each record and dump: byte 130-131 */
     for(i = 0; i<recs;i++)
     {
          fseek(data->oneb_f, (i*RECORD_SZ)+TIME_SEEK,0);
          fread(&temp[3],1,1,data->oneb_f);
          fread(&temp[2],1,1,data->oneb_f);
          fread(&temp[1],1,1,data->oneb_f);
          fread(&temp[0],1,1,data->oneb_f);
     /*
          printf("%x  ",temp[3]);
          printf("%x  ",temp[2]);
          printf("%x  ",temp[1]);
          printf("%x\n",temp[0]);
      */
          time = (((temp[1] & 0xff) << 8) | (temp[0] & 0xff));
     }

     /* Seek to the start of the packed data */
     fseek(data->oneb_f, VIDEO_SEEK,0);
     fprintf(stderr,"       [                     ] (done?)");
     for(i = 0; i<30;i++)
        fprintf(stderr,"%c",8);  /* Backup the cursor for % complete display */
     fflush(stderr);

     for(i=0; fread(unpack,sizeof(unpack),1,data->oneb_f) == 1;i++)
     {
          fill_in(data,unpack);
          fseek(data->oneb_f, ((i+1)*RECORD_SZ)+VIDEO_SEEK,0);
          if(!(i % (recs/20)))
               fprintf(stderr,"*");
     }
     fprintf(stderr,"\n\n");
     for(j = 0; j < data->ch; j++)
          fprintf(stderr,"       Actually wrote %d recs w/ %d cols per ch.\n",
                  i,2048*summ(data,j));
}

int4 summ(data,t)
onebrec *data;
int4    t;
{
     int i, s;

     s = 0;
     for(i = 0; i< 5;i++)
        s += data->raw_i[t][i];
     return s;
}

void fill_in(data,unpack)
onebrec *data;
unpack_5 *unpack;
{
     int j, k;
     unsigned char dump[10][10500];
     int4 count[10];

     bzero(count,sizeof(count));
     for(j = 0; j < COLS; j++)
     {
          for(k = 0; k < data->ch; k++)
          {
               /*
               printf("%2x\n",unpack[j].bytes.a1);
               printf("%2x\n",unpack[j].bytes.b1);
               printf("%2x\n",unpack[j].bytes.c1);
               */
               /* Do the first byte */
               if(data->raw_i[k][0])
               {
                    traceval("*      wrote   %2x\n",unpack[j].bytes.a1);
                    dump[k][count[k]++] = unpack[j].bytes.a1;
               }
               if(data->raw_i[k][1])
               {
                    traceval("*      wrote   %2x\n",unpack[j].bytes.a2);
                    dump[k][count[k]++] = unpack[j].bytes.a2;
               }
               if(data->raw_i[k][2])
               {
                    traceval("*      wrote   %2x\n",unpack[j].bytes.a3);
                    dump[k][count[k]++] = unpack[j].bytes.a3;
               }
               if(data->raw_i[k][3])
               {
                    traceval("*      wrote   %2x\n",unpack[j].bytes.a4);
                    dump[k][count[k]++] = unpack[j].bytes.a4;
               }
               if(data->raw_i[k][4])
               {
                    traceval("*      wrote   %2x\n",unpack[j].bytes.a5);
                    dump[k][count[k]++] = unpack[j].bytes.a5;
               }

               /* Do the second byte */
               if(data->raw_i[k][0])
               {
                    traceval("**     wrote   %2x\n",unpack[j].bytes.b1);
                    dump[k][count[k]++] = unpack[j].bytes.b1;
               }
               if(data->raw_i[k][1])
               {
                    traceval("**     wrote   %2x\n",unpack[j].bytes.b2);
                    dump[k][count[k]++] = unpack[j].bytes.b2;
               }
                    if(data->raw_i[k][2])
               {
                    traceval("**     wrote   %2x\n",unpack[j].bytes.b3);
                    dump[k][count[k]++] = unpack[j].bytes.b3;
               }
               if(data->raw_i[k][3])
               {
                    traceval("**     wrote   %2x\n",unpack[j].bytes.b4);
                    dump[k][count[k]++] = unpack[j].bytes.b4;
               }
               if(data->raw_i[k][4])
               {
                    traceval("**     wrote   %2x\n",unpack[j].bytes.b5);
                    dump[k][count[k]++] = unpack[j].bytes.b5;
               }

               if(j != (COLS - 1))
               {
                    /* Do the third byte */
                    if(data->raw_i[k][0])
                    {
                         traceval("***    wrote   %2x\n",unpack[j].bytes.c1);
                         dump[k][count[k]++] = unpack[j].bytes.c1;
                    }
                    if(data->raw_i[k][1])
                    {
                         traceval("***    wrote   %2x\n",unpack[j].bytes.c2);
                         dump[k][count[k]++] = unpack[j].bytes.c2;
                    }
                    if(data->raw_i[k][2])
                    {
                         traceval("***    wrote   %2x\n",unpack[j].bytes.c3);
                         dump[k][count[k]++] = unpack[j].bytes.c3;
                    }
                    if(data->raw_i[k][3])
                    {
                         traceval("***    wrote   %2x\n",unpack[j].bytes.c4);
                         dump[k][count[k]++] = unpack[j].bytes.c4;
                    }
                    if(data->raw_i[k][4])
                    {
                         traceval("***    wrote   %2x\n",unpack[j].bytes.c5);
                         dump[k][count[k]++] = unpack[j].bytes.c5;
                    }
               }
          }
     }
     for(j = 0; j < data->ch; j++)
          fwrite(dump[j],count[j],1,data->raw_f[j]);
}
