/*****************************************************************/
/*
/* gensatdat.c copyright (c) 1994 Clemson University, all rights reserved.
/*
/* GENSATDAT.C - Generate Binary Satellite Data File
/*
/*   Routine to Read Satellite Specific Spectral Constants from ASCII file*/
/*   and then generate a binary file */
/*   Modified from VAX fortran version */
/*
/* $Log: gensatdat.c,v $
 * Revision 1.1  1995/02/28  17:12:54  rheil
 * Initial revision
 *
/*  10/01/94  - Code Origination, Robert M. Heil (Bob)
/*
/******************************************************************/
/* UNIX INCLUDE FILES */
#include <stdio.h>
 
/* CREATE_1B INCLUDE FILES */
#include "macros.h"
#include "types.h"
#include "hrpt.h"

/* GLOBAL VARIABLES */
static char gensatdat_c[] = "$Header: /projects/level1b/create.1bl/C/RCS/gensatdat.c,v 1.1 1995/02/28 17:12:54 rheil Exp $";
/******************************************************************/

int4 gensatdat(rundata)
runrec      *rundata;
{
  FILE *n_ascii, *n_binary;
  rmc calibr[12]; 
  byte line[100];
  byte *lp;
  int4 i, sat, table, column, channel, temp;
  flt4 ftemp;
   
  if(rundata->d_flag)
     trace("Reading satellite data file and generating binary file ... ");
  n_ascii = fopen(rundata->ness,"r");
  n_binary = fopen(rundata->nessbin,"w");

  /*  Read in first two blank line of data file */
  fgets(line,80,n_ascii);
  fgets(line,80,n_ascii);

  /*  Read in NOAA Satellite number */
  fgets(line,80,n_ascii);
  lp = &line[5];
  sat = atoi(lp) - 1;      /* setting the array offset of rmc */
  while (atoi(lp) > 0) 
  {
      calibr[sat].id = sat+1;
  
      /*  Read in NOAA integer value */
      lp = &line[34];
      calibr[sat].inoaa = atoi(lp);
 
      /*  Read in cnoaaid string */
      calibr[sat].cnoaaid[0] = line[37];
      calibr[sat].cnoaaid[1] = line[38];

      /*  Read in text line of data file */
      fgets(line,80,n_ascii);
      fgets(line,80,n_ascii);

      /*  Read in the a(i,j) coefficients */
      for (i = 0; i<4; i++)
      {
        fscanf(n_ascii,"%f",&calibr[sat].a0[i]);
        fscanf(n_ascii,"%f",&calibr[sat].a1[i]);
        fscanf(n_ascii,"%f",&calibr[sat].a2[i]);
        fscanf(n_ascii,"%f",&calibr[sat].a3[i]);
        fscanf(n_ascii,"%f",&calibr[sat].a4[i]);
      }

      /*  Read in text line of data file */
      fgets(line,80,n_ascii);
      fgets(line,80,n_ascii);
      fgets(line,80,n_ascii);
  
      /*  Read in the b(i) thermometer temperature weights */
      for (i = 0; i<4; i++)
        fscanf(n_ascii,"%f",&calibr[sat].b[i]);

      /*  Read in text line of data file */
      fgets(line,80,n_ascii);
      fgets(line,80,n_ascii);
      fgets(line,80,n_ascii);
      fgets(line,80,n_ascii);
  
      /*  Read in Radiance of Deep Space values */
      for (i = 0; i<3; i++)
        fscanf(n_ascii,"%d%f",&temp,&calibr[sat].sp_rad[i]);

      /*  Read in text line of data file */
      fgets(line,80,n_ascii);
      fgets(line,80,n_ascii);
      fgets(line,80,n_ascii);
      fgets(line,80,n_ascii);

      /*  Read in Visible Channel Calib. Coefficients */
      for (i = 0; i<2; i++)
       fscanf(n_ascii,"%d%f%f",&temp,&calibr[sat].gain[i],&calibr[sat].xint[i]);
      for (i = 2; i<5; i++)
        calibr[sat].gain[i] = calibr[sat].xint[i] = 0;

      /*  Read in text line of data file */
      fgets(line,80,n_ascii);
      fgets(line,80,n_ascii);
      fgets(line,80,n_ascii);

      /*  Read in Non linearity error values */
      fscanf(n_ascii,"%d",&calibr[sat].num_sce_t);
      fscanf(n_ascii,"%d",&calibr[sat].num_prt_t);
      fscanf(n_ascii,"%d",&calibr[sat].num_tbls);
      for (i = 0; i<3; i++)
        fscanf(n_ascii,"%f",&calibr[sat].prt_tbl_t[i]);

      /*  Read in table(s) */
      for (table = 0; table<calibr[sat].num_tbls; table++)
        for (i = 0; i<calibr[sat].num_sce_t; i++)
        {
            fscanf(n_ascii,"%f",&calibr[sat].sce_tbl_t[i][table]);
            for (column = 0; column<calibr[sat].num_prt_t; column++)
                fscanf(n_ascii,"%f",&calibr[sat].cor_nl[i][column][table]); 
            fgets(line,80,n_ascii); /* Cleanup rest of line */
        }

      /*  Read in text line of data file */
      fgets(line,80,n_ascii);
      fgets(line,80,n_ascii);
      fgets(line,80,n_ascii);

      /*  Read Central Wave Numbers From NOAA TECH DATA BASE */
      for (i = 0; i<3; i++)
      {
        fscanf(n_ascii,"%f%f%f",&ftemp,&ftemp,&calibr[sat].cwave[0]);
        fscanf(n_ascii,"%f%f",&calibr[sat].cwave[1],&calibr[sat].cwave[2]);
      }

      /*  Read in text line of data file */
      fgets(line,80,n_ascii);
      fgets(line,80,n_ascii);
      fgets(line,80,n_ascii);
      fgets(line,80,n_ascii);
  
      /*  Read in Normalized Response Function Values */
      for (channel = 0; channel<3; channel++)
      {
       fscanf(n_ascii,"%f%f",&calibr[sat].v1[channel],&calibr[sat].dv[channel]);
       fgets(line,80,n_ascii);
       for (i = 0; i<12;i++)
           for (column = 0;column<5;column++)
               fscanf(n_ascii,"%f",&calibr[sat].rfunc[channel][column+i*5]);
       fgets(line,80,n_ascii);
       fgets(line,80,n_ascii);
       fgets(line,80,n_ascii);
      }
      if (calibr[sat].id == 7)  /*  Extra Normalized response values for #7 */
         for (i = 0;i<15;i++)
           fgets(line,80,n_ascii);
      /*  Read in NOAA Satellite number */
      fgets(line,80,n_ascii);
      lp = &line[5];
      sat = atoi(lp) - 1;      /* setting the array offset of rmc */
  }
  /*  Finished reading in the data, can now write out in a single fwrite */
  i = fwrite(calibr,sizeof(rmc),12,n_binary);
  fclose(n_binary);
  if(rundata->d_flag)
     trace("done.\n");
  return i;
}
