/*****************************************************************/
/*
/* georegister.c copyright (c) 1994 Clemson University, all rights reserved.
/*
/* GEOREGISTER.C - Setup georegistration
/*
/* $Log: georegister.c,v $
 * Revision 1.1  1995/02/28  17:12:54  rheil
 * Initial revision
 *
/*  08/01/94  - Code Origination, Walt Ligon
/*  10/10/95 -  Modified by Robert M. Heil (bob)
/*
/******************************************************************/
/* UNIX INCLUDE FILES */
#include <stdio.h>
 
/* CREATE_1B INCLUDE FILES */
#include "macros.h"
#include "types.h"
#include "hrpt.h"

/* GLOBAL VARIABLES */
static char georegister_c[] = "$Header: /projects/level1b/create.1bl/C/RCS/georegister.c,v 1.1 1995/02/28 17:12:54 rheil Exp $";

/* FUNCTIONS CALLED */
/* void browerlydanne(flt8, flt8) 
/* flt8 earthorient(int4, flt8)
/* void earthlocate(flt8, flt8, int4, flt8, flt8, flt8)
/* void initbrowerlydanne(flt4)  
/* flt8 scanorient(runrec, int4)

/* Builtin functions */
   flt8 sin(), cos(), sqrt(), atan2();
/******************************************************************/

void georegister(header, rundata)
hrpt_header *header;
runrec *rundata;
{
   int4 i;
   flt8 phig, vke[9],lat[LEN],lon[LEN];

/* initialize */
   if(rundata->d_flag)
   trace("initialzing geo parameters ... ");
   rundata->phigo = earthorient(header->pass_year,&rundata->dp);
   initbrowerlydanne(&header->semi_major_axis);
   if(rundata->d_flag)
   trace("done.\n");
/* find lat/lon for start scan */
   if(rundata->d_flag)
   trace("computing lat/lon for start scan ... ");
   phig = scanorient(rundata,rundata->starttime);
   browerlydanne(rundata->startdiff,vke);
   earthlocate(phig,vke,nsig,sigv,lat,lon);
   rundata->startlat = lat[25];
   rundata->startlon = lon[25];
   if(rundata->d_flag)
   trace("done.\n");
/* find lat/lon for end scan */
   if(rundata->d_flag)
   trace("computing lat/lon for end scan ... ");
   phig = scanorient(rundata,rundata->endtime);
   browerlydanne(rundata->enddiff,vke);
   earthlocate(phig,vke,nsig,sigv,lat,lon);
   rundata->endlat = lat[25];
   rundata->endlon = lon[25];
   if(rundata->d_flag)
   trace("done.\n");
}

flt8 earthorient(date, dp)
int4 date;
flt8 *dp;
{
   flt8 tu, phigo;
   int4 jpmod;

   /* compute years since 1900 */
   if (date < 70) date += 100;
   /* convert to days since Jan 0.5, 1900 */
   date = date * 365 + (date / 4); /* remember leap years */
   *dp = date - 0.5;
   /* now compute phigo */
   tu = *dp / 36525.0;
   phigo = (23925.836 + (8640184.542 * tu) + (0.0929 * tu * tu)) / 3600.0;
   /* truncating fraction off here */
   jpmod = phigo / 24.0;
   phigo = (phigo - (24.0 * jpmod)) * 15.0 * 3.141592654 / 180.0;
   return phigo;
}

flt8 scanorient(rundata, scantime)
runrec *rundata;
int4 scantime;
{
   flt8 dpp, epsb, omgp, dpsi, dphin;

   dpp = rundata->dp + rundata->passdate;
   epsb = 0.4093197474 - 2.271109689E-4 * (dpp / 36524.219879);
   omgp = 4.523601515 - 9.242202942E-4 * dpp;
   dpsi = -8.354648722E-5 * sin(omgp);
   dphin = dpsi * cos(epsb);
   return rundata->phigo + (1.72027912E-2 * rundata->passdate) +
               (7.292115854E-5 * (scantime / 1000.0)) + dphin;
}
