/*****************************************************************/
/*
/* initial.c copyright (c) 1994 Clemson University, all rights reserved.
/*
/* INITCAL - Routine to Read Satellite Specific Spectral Constants
/*   and then initialize Radiometric calibration variables
/*   modified from VAX fortran version
/*
/* INITHEADER - Routine to read hrpt input header file
/*
/* INITSCAN - Routine to initialize each scanline.
/*
/* $Log: initial.c,v $
 * Revision 1.2  1995/06/07  13:31:30  rheil
 * Semi Stable version for alpha, sparc, hp, and pentium.  Multiple processors.
 *
 * Revision 1.1  1995/02/28  17:12:54  rheil
 * Initial revision
 *
/*  11/14/94  - Code Modifications, Robert M. Heil (Bob)
/*
/******************************************************************/
/* UNIX INCLUDE FILES */
#include <stdio.h>
 
/* CREATE_1B INCLUDE FILES */
#include "macros.h"
#include "types.h"
#include "hrpt.h"

/* GLOBAL VARIABLES */
static char initial_c[] = "$Header: /projects/level1b/create.1bl/C/RCS/initial.c,v 1.2 1995/06/07 13:31:30 rheil Exp $";

/* FUNCTIONS CALLED */
/* void byte_swap(byte, int4, int4)
/* void dec2ieee(fltval, int4)
/* void filtp(runrec, rmc)
/* int4 gensatdat(runrec)
/* int4 get_scandate(hrpt_scanline, int2)
/* int4 get_scantime(hrpt_scanline, int2)
/* int4 julian(int4, int4, int4)
/* void space_intern(hrpt_scanline, runrec, int4, flt4, flt4)
/* void word_swap(int2, int4, int4)

/* void initcal(FILE, hrpt_header, runrec, outrec, rmc)
/* void initheader(FILE, hrpt_header, runrec)
/* void initscan(FILE, hrpt_header, runrec)
/******************************************************************/

void initcal(hrpt_file, header,rundata,outdata,cal)
FILE        *hrpt_file;
hrpt_header *header;
runrec      *rundata;
outrec      *outdata;
rmc         *cal; 
{
  FILE *data;
  hrpt_scanline scanline[75]; 
  int4 index, i, k1, k2, channel;
  int4 time, date, oldtime, olddate, deltime, deldate;
  int4 stack, prtnew[5], ready, finished, count; 
  flt4 v, sum1, sum2,  ave[4], delval1, delval2, t[4];
  flt4 tar[3], sp[3], ave10_t[3][10], ave10_s[3][10];
   
  /* Is the ascii file available ? */
  if (fopen(rundata->ness,"r") == NULL)
  {
     rundata->err_flag++;
     fprintf(stderr,"Error: cannot open file: %s\n",rundata->ness);
     exit(1);
  }

  /* Has the binary file been created? */
  if ((data = fopen(rundata->nessbin,"r")) == NULL)
  {
     if ((i = gensatdat(rundata)) > 0)
        data = fopen(rundata->nessbin,"r");
     else
     {
        fprintf(stderr,"Incorrect generation of Ness107 binary file\n");
        exit(1);
     }
  }
 
  /*  Load in the correct Satellite parameters  */
  if(rundata->d_flag)
     trace("initialzing calibration parameters ... ");
  fseek(data,(header->spacecraft_id - 1)*sizeof(rmc),0);
  fread(cal,sizeof(rmc),1,data);

  /*  Adjust values */
  for (i=0; i<2;i++)
  {
    rundata->scaled_xint[i] = cal->xint[i] * 4194304.2;
    rundata->scaled_gain[i] = cal->gain[i] * 1073741824.2;
  }

  BYTE_SWAP(rundata->scaled_xint,4,2);
  BYTE_SWAP(rundata->scaled_gain,4,2);

  if(!rundata->c_flag)
  {
    /* Pack values to output record */
    outdata->isg1 = rundata->scaled_gain[0];
    outdata->isg2 = rundata->scaled_gain[1];
    outdata->is1 = rundata->scaled_xint[0];
    outdata->is2 = rundata->scaled_xint[1];
  }

  /* Compute response weighted mean number (formerly mwave.f) */
  for (channel=0;channel<3;channel++)
  {
    v = cal->v1[channel];
    sum1 = sum2 = 0;
    for (i=0;i<60;i++)
    {
      sum1 += cal->rfunc[channel][i];
      sum2 += cal->rfunc[channel][i] * v;
      v    += cal->dv[channel];
    }
    rundata->vmean[channel] = sum2/sum1;
  }
  
  /* Collect 50 good input data records */
  fseek(hrpt_file, HDRSZ+SCANSZ*rundata->firstscan,0);
  fread(scanline, SCANSZ, 75, hrpt_file); /* grab some extra scans */
  /*BYTE_SWAP(scanline[0],2,(SCANSZ/2)*(75)); */
  oldtime = olddate = index = ready = stack = finished = 0; 
  while (!finished)
  {
      time = get_scantime(&scanline[index],1);
      date = get_scandate(&scanline[index],1);
      deltime = time - oldtime;
      deldate = date - olddate;
      if (deldate == 0 && (deltime == 166 || deltime == 167)) /* good scan */
      {
         /* PRT value from minor frame header */
         BYTE_SWAP(&scanline[index].doc.telemetry[2],2,1);
         if (scanline[index].doc.telemetry[2] <= 10)
         {
            rundata->prt[0][stack] = 1;
            ready = 1;
            count = 0;
         }
         if (ready && scanline[index].doc.telemetry[2] > 10 && count < 4)
            rundata->prt[++count][stack]=scanline[index].doc.telemetry[2];
         if (ready && count == 4) /* Can immediately increment stack! */
         {
            if (++stack == (10))
               finished = 1;
            ready = 0;
         }

         /* Internal Target data & Space view counts */
         BYTE_SWAP(&scanline[index].doc.internal[0].ch[0],2,80); /* 30 + 50 */
         space_intern(&scanline[index],rundata,stack,tar,sp);
         for (channel=0;channel<3;channel++)
         {
             ave10_t[channel][stack-1] = tar[channel];
             ave10_s[channel][stack-1] = sp[channel];
         }
      }
      oldtime = time;
      olddate = date;
      index++;
  }
  /* Compute 100 sample mean values for space and internal target count */
  for (channel=0;channel<3;channel++)
  {
      sum1 = sum2 = 0;
      for (i=0;i<10;i++)
      {
          sum1 += ave10_t[channel][i];
          sum2 += ave10_s[channel][i];
      }
      rundata->tarave[channel] = sum1/(10);
      rundata->spave[channel] = sum2/(10);
      if (rundata->tarave[channel] >= rundata->spave[channel])
         fprintf(stderr,"IRINIT - warning: Invalid target data\n");
  }
  /*  Compute internal target temperatures */
  filtp(rundata,cal);
      if (rundata->tempave <= 0)
      {
          fprintf(stderr,"IRINIT - error: Average PRT temp <= 0!\n");
          exit(1);
      }
  if(rundata->d_flag)
     trace("done.\n");
}

initheader(hrpt_file,header,rundata)
FILE *hrpt_file;
hrpt_header *header;
runrec *rundata;
{
     /* find file size */
     fseek(hrpt_file,0,2);
     rundata->scanlines = (ftell(hrpt_file) / 512 - 1) / 43;
     /* read file header */
     fseek(hrpt_file,0,0);
     fread(header,HDRSZ,1,hrpt_file);
     BYTE_SWAP(header,4,128);
     WORD_SWAP(&header->DCS_second,2,1);
     WORD_SWAP(&header->epoch_second,2,7);
     DEC2IEEE(&header->DCS_second,1);
     DEC2IEEE(&header->epoch_second,7);
     /* generate julian dates */
     rundata->epochdate = julian(header->epoch_year,header->epoch_month,
          header->epoch_day);
     rundata->passdate = julian(header->program_year,header->program_month,
          header->program_day);
     if (header->pass_year != header->epoch_year)
     {
          fprintf(stderr,"pass year does not equal epoch year\n");
          exit(1);
     }
     rundata->epochsec = (rundata->epochdate*86400.0)+
          (header->epoch_hour*3600.0)+ (header->epoch_minute*60.0)+
          header->epoch_second;
     /* check to see if data should be updated from tbus */
     /* updatetbus(); */
     /* convert degrees to radians */
     header->inclination /= DEGRAD;
     header->ascenting_node_angle /= DEGRAD;
     header->angle_of_perigee /= DEGRAD;
     header->mean_anomally /= DEGRAD;
}

initscan(hrpt_file,header,rundata)
FILE *hrpt_file;
hrpt_header *header;
runrec *rundata;
{
	hrpt_scanline scanline;
	int4 index = 0;
	int4 oldtime, scantime, timediff;

     if(rundata->d_flag)
	   trace("searching for last good scan ... ");
	timediff = 0;
	oldtime = scantime = 0;
	for (index = rundata->scanlines - 3;
		timediff < 166 || timediff > 167 ; index--)
	{
		fseek(hrpt_file,HDRSZ+SCANSZ*index,0);
		fread(&scanline,BLKSZ,1,hrpt_file);
		oldtime = scantime;
		scantime = get_scantime(&scanline,1);
		timediff = oldtime - scantime;
	}
	rundata->lastscan = index + 2;
	rundata->endtime = scantime;
	rundata->enddate = get_scandate(&scanline,1);
	rundata->endsec = rundata->enddate*86400.0+rundata->endtime/1000.0;
	rundata->enddiff = rundata->endsec - rundata->epochsec;
	if (rundata->enddate == 365 && rundata->enddiff < 0)
		rundata->enddiff += 31536000.0;
     if(rundata->d_flag)
	   trace("done.\n");

     if(rundata->d_flag)
	   trace("searching for first good scan ... ");
	timediff = 0;
	oldtime = 0;
	fseek(hrpt_file,HDRSZ,0);
	for (index = 1; timediff < 166 || timediff > 167 ; index++)
	{
		fread(&scanline,BLKSZ,1,hrpt_file);
		fseek(hrpt_file,SCANSZ*index-BLKSZ,1);
		oldtime = scantime;
		scantime = get_scantime(&scanline,1);
		timediff = scantime - oldtime;
	}
	rundata->firstscan = index - 2;
	rundata->starttime = oldtime;
	rundata->startdate = get_scandate(&scanline,1);
	rundata->startsec = rundata->startdate*86400.0+rundata->starttime/1000.0;
	rundata->startdiff = rundata->startsec - rundata->epochsec;
	if (rundata->startdate == 365 && rundata->startdiff < 0)
		rundata->startdiff += 31536000.0;
     if(rundata->d_flag)
	   trace("done.\n");

     /* this does NOT work, yields a-a = 0;
	rundata->ngaps = rundata->scanlines - (1.0 + 6.0 *
		(rundata->endtime - rundata->starttime) / 1000.0);
     */
     rundata->ngaps = 0; /* Will fill in at end of program */
	rundata->nscans = rundata->lastscan - rundata->firstscan + 1;

	if (rundata->startdate - rundata->epochdate > 5)
	{
		fprintf(stderr,"Warning: epoch date more than 5 days old\n");
	}

	if (rundata->startdate < rundata->epochdate)
	{
		fprintf(stderr,"epoch date inconsistent with scan date\n");
		exit(1);
	}
}
