#ifndef MACROS_H
#define MACROS_H

/*****************************************************************/
/*
/* macros.h copyright (c) 1995 Clemson University, all rights reserved.
/*
/* MACROS.H - Header file that sets certain flags and generates machine 
/*            dependent information
/*
/* $Log: macros.h,v $
 * Revision 1.3  1995/06/07  13:31:30  rheil
 * Semi Stable version for alpha, sparc, hp, and pentium.  Multiple processors.
 *
 * Revision 1.2  1995/03/10  16:24:37  rheil
 * Added header and made compile-time flags consistent
 * /
 *
/*
/******************************************************************/

/* GLOBAL VARIABLES */
static char macros_h[] = "$Header: /projects/level1b/create.1bl/C/RCS/macros.h,v 1.3 1995/06/07 13:31:30 rheil Exp $";

#ifdef  DEBUG
#define trace(x) printf(x)
#define traceval(x,y) printf(x,y)
#else
#define trace(x) 
#define traceval(x,y) 
#endif

#ifdef HBF
#define EXPONENT 2
#define BYTE_SWAP(p,s,c) byte_swap(p,s,c)
typedef struct fltval
{
     unsigned int sign:1;
     unsigned int exponent:8;
     unsigned int mantissa:23;
} fltval;
typedef struct pack_3
{
    unsigned int sp  :2;
    unsigned int cha :10;
    unsigned int chb :10;
    unsigned int chc :10;
    } pack_3;

typedef union unpack_5
{
     struct bytes
     {
          unsigned int t1  : 2;
          unsigned int a1  : 8;
          unsigned int s1  : 2;
          unsigned int a2  : 8;
          unsigned int s2  : 2;
          unsigned int a3  : 8;
          unsigned int s3  : 2;
          
          unsigned int t2  : 2;
          unsigned int a4  : 8;
          unsigned int s4  : 2;
          unsigned int a5  : 8;
          unsigned int s5  : 2;
          unsigned int b1  : 8;
          unsigned int s6  : 2;
          
          unsigned int t3  : 2;
          unsigned int b2  : 8;
          unsigned int s7  : 2;
          unsigned int b3  : 8;
          unsigned int s8  : 2;
          unsigned int b4  : 8;
          unsigned int s9  : 2;
          
          unsigned int t4  : 2;
          unsigned int b5  : 8;
          unsigned int s10 : 2;
          unsigned int c1  : 8;
          unsigned int s11 : 2;
          unsigned int c2  : 8;
          unsigned int s12 : 2;
          
          unsigned int t5  : 2;
          unsigned int c3  : 8;
          unsigned int s13 : 2;
          unsigned int c4  : 8;
          unsigned int s14 : 2;
          unsigned int c5  : 8;
          unsigned int s15 : 2;
     } bytes;

     struct ten_bits
     {
          unsigned int s1  : 2;
          unsigned int a1  : 10;
          unsigned int a2  : 10;
          unsigned int a3  : 10;

          unsigned int s2  : 2;
          unsigned int a4  : 10;
          unsigned int a5  : 10;
          unsigned int b1  : 10;

          unsigned int s3  : 2;
          unsigned int b2  : 10;
          unsigned int b3  : 10;
          unsigned int b4  : 10;

          unsigned int s4  : 2;
          unsigned int b5  : 10;
          unsigned int c1  : 10;
          unsigned int c2  : 10;

          unsigned int s5  : 2;
          unsigned int c3  : 10;
          unsigned int c4  : 10;
          unsigned int c5  : 10;
     } ten_bits;
} unpack_5;
#else
#define EXPONENT  1
#define BYTE_SWAP(p,s,c)
typedef struct fltval
{
     unsigned int mantissa:23;
     unsigned int sign:1;
     unsigned int exponent:8;
} fltval;
typedef struct pack_3
{
    unsigned int chc :10;
    unsigned int chb :10;
    unsigned int cha :10;
    unsigned int sp  :2;
} pack_3;

typedef union unpack_5
{
     struct bytes
     {
          unsigned int s1  : 2;
          unsigned int a3  : 8;
          unsigned int s2  : 2;
          unsigned int a2  : 8;
          unsigned int s3  : 2;
          unsigned int a1  : 8;
          unsigned int t1  : 2;
          
          unsigned int s4  : 2;
          unsigned int b1  : 8;
          unsigned int s5  : 2;
          unsigned int a5  : 8;
          unsigned int s6  : 2;
          unsigned int a4  : 8;
          unsigned int t2  : 2;
          
          unsigned int s7  : 2;
          unsigned int b4  : 8;
          unsigned int s8  : 2;
          unsigned int b3  : 8;
          unsigned int s9  : 2;
          unsigned int b2  : 8;
          unsigned int t3  : 2;
          
          unsigned int s10 : 2;
          unsigned int c2  : 8;
          unsigned int s11 : 2;
          unsigned int c1  : 8;
          unsigned int s12 : 2;
          unsigned int b5  : 8;
          unsigned int t4  : 2;
          
          unsigned int s13 : 2;
          unsigned int c5  : 8;
          unsigned int s14 : 2;
          unsigned int c4  : 8;
          unsigned int s15 : 2;
          unsigned int c3  : 8;
          unsigned int t5  : 2;
     } bytes;

     struct ten_bits
     {
          unsigned int a3  : 10;
          unsigned int a2  : 10;
          unsigned int a1  : 10;
          unsigned int s1  : 2;

          unsigned int b1  : 10;
          unsigned int a5  : 10;
          unsigned int a4  : 10;
          unsigned int s2  : 2;

          unsigned int b4  : 10;
          unsigned int b3  : 10;
          unsigned int b2  : 10;
          unsigned int s3  : 2;

          unsigned int c2  : 10;
          unsigned int c1  : 10;
          unsigned int b5  : 10;
          unsigned int s4  : 2;

          unsigned int c5  : 10;
          unsigned int c4  : 10;
          unsigned int c3  : 10;
          unsigned int s5  : 2;
     } ten_bits;
} unpack_5;
#endif
#define WORD_SWAP(p,s,c) word_swap(p,s,c)

#ifdef DECFP
#define DEC2IEEE(p,c)
#else
#define DEC2IEEE(p,c) dec2ieee(p,c)
#endif

#ifdef TIMING
#define GETTIMEOFDAY(p1, p2) gettimeofday(p1, p2)
#else
#define GETTIMEOFDAY(p1, p2)
#endif

#define SCANSZ sizeof(hrpt_scanline)
#define HDRSZ  sizeof(hrpt_header)
#define OUTSZ  sizeof(outrec)
#define BLKSZ  512
#define LEN    51

#define SCAN_CNT 3414 /* number of scan records to generate */
#define CH1 0       /* starting offset of channel 1 data */
#define CH2 2048      /* starting offset of channel 2 data */
#define CH3 4096      /* starting offset of channel 3 data */
#define CH4 6144      /* starting offset of channel 4 data */
#define CH5 8192      /* starting offset of channel 5 data */

#define DEGRAD	57.29577951308238

static double sigv[] = {
     -54.00,-51.84,-49.68,-47.52,-45.36,-43.20,-41.04,-38.88,-36.72,
     -34.56,-32.40,-30.24,-28.08,-25.92,-23.76,-21.60,-19.44,-17.28,
     -15.12,-12.96,-10.80,-08.64,-06.48,-04.32,-02.16, 00.00, 02.16,
      04.32, 06.48, 08.64, 10.80, 12.96, 15.12, 17.28, 19.44, 21.60,
      23.76, 25.92, 28.08, 30.24, 32.40, 34.56, 36.72, 38.88, 41.04,
      43.20, 45.36, 47.52, 49.68, 51.84, 54.0};
static int nsig = sizeof(sigv)/sizeof(double);

#endif
