/*****************************************************************/
/*
/* radiance.c copyright (c) 1994 Clemson University, all rights reserved.
/*
/* RADIANCE.C - This Routine Computes Radiance Of Internal Target            
/*  Based On Prt Measured Ave Temperature.               
/* 
/*  An Average Black Body Radiance Is Obtained For Each  
/*  Band By Weighting The Planck Function With The       
/*  Normalized Spectral Response Across The Spectral     
/*  Band Width Of A Channel.                             
/* 
/*  Rfunc(i) = Response Function                           
/*  V        = 1/wavelenth (1/cm)                          
/*
/* $Log: radiance.c,v $
 * Revision 1.2  1995/03/10  16:25:49  rheil
 * Added math library to insure proper function returns
 *
 * Revision 1.1  1995/02/28  17:12:54  rheil
 * Initial revision
 *
/*  10/20/94  - Code Modification, Robert M. Heil (Bob)
/*
/******************************************************************/
/* UNIX INCLUDE FILES */
#include <stdio.h>
#include <math.h>
 
/* CREATE_1B INCLUDE FILES */
#include "macros.h"
#include "types.h"

/* GLOBAL VARIABLES */
static char radiance_c[] = "$Header: /projects/level1b/create.1bl/C/RCS/radiance.c,v 1.2 1995/03/10 16:25:49 rheil Exp $";
/******************************************************************/
                                      
void radiance(rundata,cal)
runrec *rundata;
rmc    *cal;
{

    int4  channel, i;
    flt4 sum, v, temp[3];

    /* Sum Response Function Over The Band */ 
    for (channel=0;channel<3;channel++)
    {
        temp[channel] = 0;
        for (i=0;i<60;i++)
          temp[channel] += cal->rfunc[channel][i];
    }

    /* Compute Weighted Black Body Radiance */
    for (channel=0;channel<3;channel++)
    {
        sum = 0;
        v   = cal->v1[channel]; 
        for (i=0;i<60;i++)
        {
            sum += cal->rfunc[channel][i] * 1.1910659e-05 * v * v * v / 
                   (exp (1.438833 * v / rundata->tempave) - 1);
            v += cal->dv[channel];
        }
        rundata->rad[channel] = sum/temp[channel];
    }

    /* Compute gain factors and intercepts */
    for (i=0;i<3;i++)
    {
            cal->gain[i+2] = (cal->sp_rad[i] - rundata->rad[i]) / 
                             (rundata->spave[i] - rundata->tarave[i]);
            cal->xint[i+2] = cal->sp_rad[i] - cal->gain[i+2]*rundata->spave[i];
    }
}
