/*****************************************************************/
/*
/* swap_pack.c copyright (c) 1994 Clemson University, all rights reserved.
/*
/* SWAP_PACK.C - Routines to pack data, swap data, and other various
/*               stuff.
/*
/* $Log: swap_pack.c,v $
 * Revision 1.3  1995/06/07  13:31:30  rheil
 * Semi Stable version for alpha, sparc, hp, and pentium.  Multiple processors.
 *
 * Revision 1.2  1995/04/07  14:31:21  rheil
 * replaced uints with unsigned int etc.
 *
 * Revision 1.1  1995/02/28  17:12:54  rheil
 * Initial revision
 *
/*  01/10/95 - Grouping of code together.
/*
/******************************************************************/
/* UNIX INCLUDE FILES */
#include <stdio.h>
 
/* CREATE_1B INCLUDE FILES */
#include "macros.h"
#include "types.h"
#include "hrpt.h"
#include <sys/time.h>

/* GLOBAL VARIABLES */
static char swap_pack_c[] = "$Header: /projects/level1b/create.1bl/C/RCS/swap_pack.c,v 1.3 1995/06/07 13:31:30 rheil Exp $";

/* FUNCTIONS INCLUDED */
/* void byte_swap(byte, int4, int4)
/* void dec2ieee(fltval, int4)
/* void earth_pack(flt8, flt8, outrec)
/* int4 get_scandate(hrpt_scanline, int2)
/* int4 get_scantime(hrpt_scanline, int2)
/* int4 julian(int4, int4, int4)
/* void quality_pack(hrpt_scanline, hrpt_header, runrec, rmc, qual)
/* void telem_pack(int2, pack_3)
/* void video_pack(int2, pack_3)
/* void word_swap(int2, int4, int4)
/******************************************************************/

byte_swap(data, size, count)
byte data[];
int4 size;
int4 count;
{
     int4 num_swaps = size/2;
     int4 swaps, partner;
     while(count--)
     {
          for (swaps = 0, partner = size-1;
               swaps < num_swaps; swaps++, partner--)
          {
               char tmp = data[swaps];
               data[swaps] = data[partner];
               data[partner] = tmp;
          }
          data += size;
     }
}
/******************************************************************/

word_swap(data, size, count)
int2 data[];
int4 size;
int4 count;
{
     int4 num_swaps = size/2;
     int4 swaps, partner;
     int2 tmp;

     while(count--)
     {
          for (swaps = 0, partner = size-1;
               swaps < num_swaps; swaps++, partner--)
          {
               tmp = data[swaps];
               data[swaps] = data[partner];
               data[partner] = tmp;
          }
          data += size;
     }
}
/******************************************************************/

dec2ieee(value, count)
fltval *value;
int4 count;
{
    for (; count--; value++)
        value->exponent -= EXPONENT;
}
/******************************************************************/
 
void earth_pack(lat,lon,outdata)
flt8 *lat,*lon;
outrec *outdata;
{
    int2 la[LEN],lo[LEN];
    int4 i;

    outdata->igood = LEN;
    for (i=0;i<LEN;i++)
    {
        if((lat[i]> 90.0)||(lat[i]< -90.0)||(lon[i]> 180.0)||(lon[i]< -180.0))
            outdata->igood--;
        la[i] = (int) (lat[i] * 128.0);
        lo[i] = (int) (lon[i] * 128.0);
        outdata->iearth[i] = ((lo[i] & 0x00ff) <<24) | ((lo[i] & 0xff00) <<8) |
                             ((la[i] & 0x00ff) <<8) | ((la[i] & 0xff00) >>8); 
    }
}
/******************************************************************/

get_scantime(scanline,swap)
hrpt_scanline *scanline;
int2          swap;
{
	register int2 *p = &scanline->doc.time[1];
     int4 time;

	BYTE_SWAP(p,2,3);
	time = ((p[0] & 0x7f) << 20) | ((p[1] & 0x3ff) << 10) | (p[2] & 0x3ff);
     if(!swap) /* if need to fix the swapped info */
	   BYTE_SWAP(p,2,3);
     return time;
	}
/******************************************************************/

get_scandate(scanline,swap)
hrpt_scanline *scanline;
int2          swap;
{
	register int2 *p = &scanline->doc.time[0];
     int4 date;

	BYTE_SWAP(p,2,1);
	date = (p[0] >> 1) & 0x1ff;
     if(!swap)
	   BYTE_SWAP(p,2,1);
     return date;
	}
/******************************************************************/

julian(year, month, day)
int4 year;
int4 month;
int4 day;
{
	static int4 jdays[12] = {0,31,59,90,120,151,181,212,243,273,304,334};
	return day + jdays[month-1] + (year % 4 && month > 2 ? 0 : 1);
	}
/******************************************************************/

quality_pack(scanline, header, rundata, cal, q_bit)
hrpt_scanline *scanline;
hrpt_header   *header;
runrec        *rundata;
rmc           *cal;
qual          *q_bit;
{

    int2 temp;
    int2 i;

    /* Fatal Flag?           Bit 31 (formerly ff.f)    */
    q_bit->fe = 0;
    
    /* Time or Date error?   Bit 30 (formerly tim.f)   */
    q_bit->dte = 0;

    /* Data Gap?             Bit 29 (formerly dg.f)    */
    q_bit->dg = 0;

    /* Data Jitter?          Bit 28 (formerly dj.f)    */
    /* 0 indicates stable frame, 1 indicates a resync occurred */
    q_bit->dj = ((scanline->doc.id[0] >> 2) & 0x0001);

    /* Calibration Factors?  Bit 27 (formerly cal.f)   */
    temp = 0;
    for (i=2;i<5;i++)
    {
       if ((cal->xint[i] > 1e8) || (cal->xint[i] < -1e8))
           temp = 1;
       if ((cal->gain[i] > 1e10) || (cal->gain[i] < -1e10))
           temp = 1;
    }
    q_bit->ce = temp & 0x0001;

    /* Earth Loc data?       Bit 26 (formerly el.f)    */
    if (header->semi_major_axis == 0.0)
        q_bit->ee = 1;
    else
        q_bit->ee = 0;

    /* Ascending/Descending? Bit 25 (formerly asde.f)  */
    /* 0 indicates ascending, 1 indicates descending */
    q_bit->ad = (header->orbital_node - 1) & 0x0001;
}
/******************************************************************/

telem_pack(raw, packed1b)
int2  *raw;
pack_3 *packed1b;
{
    int4 i;
    for(i = 0 ;i < 30; i++)
    {
        packed1b->sp  = 0;
        packed1b->cha = *raw++;
        packed1b->chb = *raw++;
        packed1b->chc = *raw++;
        packed1b++;
    }

}

video_pack(raw, packed1b)
int2 *raw;
pack_3 *packed1b;
{
    int2 *raw_ch1_ptr = &raw[CH1];
    int2 *raw_ch2_ptr = &raw[CH2];
    int2 *raw_ch3_ptr = &raw[CH3];
    int2 *raw_ch4_ptr = &raw[CH4];
    int2 *raw_ch5_ptr = &raw[CH5];
    unsigned int scan_recs;
    for(scan_recs = 0 ;scan_recs < SCAN_CNT; scan_recs += 5)
    {
        /*
        printf("%2x\n",((*raw_ch1_ptr & 0x03fc) >> 2));
        */
        packed1b->cha = *raw_ch1_ptr++;
        packed1b->chb = *raw_ch2_ptr++;
        packed1b->chc = *raw_ch3_ptr++;
        packed1b->sp  = 0;
        packed1b++;

        /*
        printf("%2x\n",((*raw_ch1_ptr & 0x03fc) >> 2));
        */
        packed1b->cha = *raw_ch4_ptr++;
        packed1b->chb = *raw_ch5_ptr++;
        packed1b->chc = *raw_ch1_ptr++;
        packed1b->sp  = 0;
        packed1b++;

        packed1b->cha = *raw_ch2_ptr++;
        packed1b->chb = *raw_ch3_ptr++;
        packed1b->chc = *raw_ch4_ptr++;
        packed1b->sp  = 0;
        packed1b++;

        /*
        printf("%2x\n",((*raw_ch1_ptr & 0x03fc) >> 2));
        */
        packed1b->cha = *raw_ch5_ptr++;
        packed1b->chb = *raw_ch1_ptr++;
        packed1b->chc = *raw_ch2_ptr++;
        packed1b->sp  = 0;
        packed1b++;

        packed1b->cha = *raw_ch3_ptr++;
        packed1b->chb = *raw_ch4_ptr++;
        packed1b->chc = *raw_ch5_ptr++;
        packed1b->sp  = 0;
        packed1b++;
    }
}
