
#ifndef TYPES_H
#define TYPES_H

/*****************************************************************/
/*
/* types.h copyright (c) 1995 Clemson University, all rights reserved.
/*
/* TYPES.H - Header file that sets variable sizes, and defines various 
/*           structures used in create1b
/*
/* $Log: types.h,v $
 * Revision 1.4  1995/06/07  13:31:30  rheil
 * Semi Stable version for alpha, sparc, hp, and pentium.  Multiple processors.
 *
 * Revision 1.3  1995/04/07  14:31:21  rheil
 * replaced uints with unsigned int etc.
 *
 * Revision 1.2  1995/03/10  16:24:37  rheil
 * Added header and made compile-time flags consistent
 * /
 *
/*
/******************************************************************/

/* GLOBAL VARIABLES */
static char types_h[] = "$Header: /projects/level1b/create.1bl/C/RCS/types.h,v 1.4 1995/06/07 13:31:30 rheil Exp $";

typedef char	*addr;
typedef char	byte;
typedef char	int1;
typedef short	int2;
typedef int	int4;
#ifdef DBLWD
typedef long   int8;
#endif
typedef float	flt4;
typedef double	flt8;


#define CALMIN 50  /* Minimum set of calibration data (Multiple of 5) */

typedef struct runrec
{
     /*   OUTFILE holds file name(s) of packed(1)/unpacked(1-5) data sets */
     /*   NESS holds file name of satellite data file */
     /*   NESSBIN holds file name of binary packed satellite data file */
     /*   I, N, O, G, C, D, V, U, ERR are command line flags set @ execution */
     /*   CHANNELS indicates the number of output channels being written(1-5)*/
     /*   SCANLINES is the number of scan lines in input file */

     byte outfile[6][52], ness[52], nessbin[52];
     int4 i_flag, n_flag, o_flag, g_flag, c_flag, d_flag, v_flag, u_flag;
     int4 x_flag, y_flag, r_flag, z_flag, err_flag;
     int4 channels, scanlines, nscans, ngaps;
     int4 firstscan, lastscan, startdate, starttime, enddate, endtime;
     int4 passdate, epochdate, scaled_xint[5], scaled_gain[5];
     flt4 vmean[3], tempave, newprt[5], prt[5][10], xtarget[3][10];
     flt4 xspace[3][10], tarave[3], spave[3], rad[3]; 
     flt8 startsec, startdiff, endsec, enddiff;
     flt8 startlat, startlon, endlat, endlon;
     flt8 epochsec, phigo, dp;
     } runrec;

typedef struct rmc  /* Radiometric Calibration */
{
     int4 id, inoaa;
     byte cnoaaid[2];
     flt4 a0[4], a1[4], a2[4], a3[4], a4[4], b[4], sp_rad[3], gain[5], xint[5];
     int4 num_sce_t, num_prt_t, num_tbls;
     flt4 prt_tbl_t[3], sce_tbl_t[20][2], cor_nl[20][3][2], cwave[3];
     flt4 v1[3], dv[3], rfunc[3][60];
     }rmc;

typedef struct onebrec  /* Radiometric Calibration */
{
     int4 inf, er, outf, ch, w;
     FILE *oneb_f;
     FILE *raw_f[10];
     int4 raw_i[10][5];

     char raw_n[10][20];
     char suffix[10][10];
     char oneb_n[20];

     }onebrec;

typedef struct outrec  /* Level 1b Formatted output video and misc */
{
     int2 inum, jdat;          /* Scan Number, date, and time */
     int4 jtim, iqual;
     int4 isg1, is1;           /* Calibration factors */
     int4 isg2, is2;
     int4 isg3, is3;
     int4 isg4, is4;
     int4 isg5, is5;
     unsigned char igood, izen[51];    /* Registration, quality, and telemetry data */
     int4 iearth[51], itelm[30];
     int4 filler[5];
     int4 iscan[3500];         /* Video data */
     int4 spare[88];
     }outrec;                  /* Total length should be 14800 bytes */

typedef struct qual
{
    unsigned int fe  :1;  /* Fatal error               */
    unsigned int dte :1;  /* Date/Time error           */
    unsigned int dg  :1;  /* Data gap error            */
    unsigned int dj  :1;  /* Data Jitter               */
    unsigned int ce  :1;  /* Calibration factor error  */
    unsigned int ee  :1;  /* Earth location data error */
    unsigned int ad  :1;  /* Ascending/Descending      */
    unsigned int     :25; /* Rest of the quality bits  */
    } qual;

flt8 earthorient();
flt8 scanorient();

#endif
